#!/bin/bash
#
# Makes a sysroot from a Ubuntu release.  Tries to follow the same
# style as multistrap.
#
set -e

# Configuration

# Suite to fetch (natty, oneiric, testing, ...)
suite=${1:-oneiric}
arch=${2:-armel}
# Mirror to fetch from
source=http://mirror.aarnet.edu.au/pub/ubuntu-ports/
components="main universe"
packages="libc6 libc6-dev libc6-dbg linux-libc-dev zlib1g"
roots="xaos gnome-games"
exclude="libgcc* libstdc++*"

# Locals

build=build
# Where to fetch patches
dist=$build/dist
# Where to build the temporary sysroot
sysroot=$build/sysroot

etc=$dist/$suite/etc
archives=$dist/$suite/var/cache/apt/archives

# Start afresh
rm -rf $build
mkdir -p $dist $sysroot

# Create the distro
chdist -d $dist -a $arch create $suite
# Make a couple of directories to supress APT warnings
mkdir -p $etc/apt/{apt.conf.d,preferences.d}

# Create the source list and update
cat > $etc/apt/sources.list <<EOF
deb $source $suite $components
deb-src $source $suite $components
EOF

chdist -d $dist apt-get $suite update

# Grab all of the packages
chdist -d $dist apt-get $suite install -dyq --no-install-recommends $packages

# And the build-deps of the roots
if [ -n "$roots" ]; then
    chdist -d $dist apt-get $suite build-dep -dyq --no-install-recommends $roots
fi

# Remove any troublesome packages
for i in $exclude; do
    rm -vf $archives/$i.deb
done

# Extract the packages that we want and all devel-like packages
for i in $packages "lib*" "*-dev"; do
    for j in $archives/${i}_*.deb; do
        echo Extracting $( basename $j )
        dpkg-deb -x $j $sysroot
    done
done

# Tidy up
rm -rf $sysroot/etc

# Some packages include absolute links in sysroot/usr/lib.
# Convert to relative links instead
for lib in $( find $sysroot -type l ); do
    target=$( readlink $lib )
    base=$( basename $target )

    case $target in
        /*)
            v=$( echo $lib | sed -r "s#^$sysroot/##" | tr -cd / | sed "s#/#../#g" )
            p=$( echo $target | sed "s#^/##" )
            rm $lib
            ln -s $v$p $lib
            ;;
        *)  ;;
    esac
done

# Tarball it up
tar caf $suite-sysroot.tar.bz2 -C $build sysroot
